@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 2000-2001  Qualcomm Incorporated
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2007  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2008  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -317,6 +317,7 @@ static hci_map commands_map[] = {
 	{ "Disconnect",					5   },
 	{ "Add SCO Connection",				6   },
 	{ "Cancel Create Connection",			7   },
+
 	{ "Accept Connection Request",			8   },
 	{ "Reject Connection Request",			9   },
 	{ "Link Key Request Reply",			10  },
@@ -325,6 +326,7 @@ static hci_map commands_map[] = {
 	{ "PIN Code Request Negative Reply",		13  },
 	{ "Change Connection Packet Type",		14  },
 	{ "Authentication Requested",			15  },
+
 	{ "Set Connection Encryption",			16  },
 	{ "Change Connection Link Key",			17  },
 	{ "Master Link Key",				18  },
@@ -333,6 +335,7 @@ static hci_map commands_map[] = {
 	{ "Read Remote Supported Features",		21  },
 	{ "Read Remote Extended Features",		22  },
 	{ "Read Remote Version Information",		23  },
+
 	{ "Read Clock Offset",				24  },
 	{ "Read LMP Handle",				25  },
 	{ "Reserved",					26  },
@@ -341,6 +344,7 @@ static hci_map commands_map[] = {
 	{ "Reserved",					29  },
 	{ "Reserved",					30  },
 	{ "Reserved",					31  },
+
 	{ "Reserved",					32  },
 	{ "Hold Mode",					33  },
 	{ "Sniff Mode",					34  },
@@ -349,6 +353,7 @@ static hci_map commands_map[] = {
 	{ "Exit Park State",				37  },
 	{ "QoS Setup",					38  },
 	{ "Role Discovery",				39  },
+
 	{ "Switch Role",				40  },
 	{ "Read Link Policy Settings",			41  },
 	{ "Write Link Policy Settings",			42  },
@@ -357,6 +362,7 @@ static hci_map commands_map[] = {
 	{ "Flow Specification",				45  },
 	{ "Set Event Mask",				46  },
 	{ "Reset",					47  },
+
 	{ "Set Event Filter",				48  },
 	{ "Flush",					49  },
 	{ "Read PIN Type",				50  },
@@ -365,6 +371,7 @@ static hci_map commands_map[] = {
 	{ "Read Stored Link Key",			53  },
 	{ "Write Stored Link Key",			54  },
 	{ "Delete Stored Link Key",			55  },
+
 	{ "Write Local Name",				56  },
 	{ "Read Local Name",				57  },
 	{ "Read Connection Accept Timeout",		58  },
@@ -373,6 +380,7 @@ static hci_map commands_map[] = {
 	{ "Write Page Timeout",				61  },
 	{ "Read Scan Enable",				62  },
 	{ "Write Scan Enable",				63  },
+
 	{ "Read Page Scan Activity",			64  },
 	{ "Write Page Scan Activity",			65  },
 	{ "Read Inquiry Scan Activity",			66  },
@@ -381,6 +389,7 @@ static hci_map commands_map[] = {
 	{ "Write Authentication Enable",		69  },
 	{ "Read Encryption Mode",			70  },
 	{ "Write Encryption Mode",			71  },
+
 	{ "Read Class Of Device",			72  },
 	{ "Write Class Of Device",			73  },
 	{ "Read Voice Setting",				74  },
@@ -389,6 +398,7 @@ static hci_map commands_map[] = {
 	{ "Write Automatic Flush Timeout",		77  },
 	{ "Read Num Broadcast Retransmissions",		78  },
 	{ "Write Num Broadcast Retransmissions",	79  },
+
 	{ "Read Hold Mode Activity",			80  },
 	{ "Write Hold Mode Activity",			81  },
 	{ "Read Transmit Power Level",			82  },
@@ -397,6 +407,7 @@ static hci_map commands_map[] = {
 	{ "Set Host Controller To Host Flow Control",	85  },
 	{ "Host Buffer Size",				86  },
 	{ "Host Number Of Completed Packets",		87  },
+
 	{ "Read Link Supervision Timeout",		88  },
 	{ "Write Link Supervision Timeout",		89  },
 	{ "Read Number of Supported IAC",		90  },
@@ -405,14 +416,16 @@ static hci_map commands_map[] = {
 	{ "Read Page Scan Period Mode",			93  },
 	{ "Write Page Scan Period Mode",		94  },
 	{ "Read Page Scan Mode",			95  },
+
 	{ "Write Page Scan Mode",			96  },
 	{ "Set AFH Channel Classification",		97  },
-	{ "reserved",					98  },
-	{ "reserved",					99  },
+	{ "Reserved",					98  },
+	{ "Reserved",					99  },
 	{ "Read Inquiry Scan Type",			100 },
 	{ "Write Inquiry Scan Type",			101 },
 	{ "Read Inquiry Mode",				102 },
 	{ "Write Inquiry Mode",				103 },
+
 	{ "Read Page Scan Type",			104 },
 	{ "Write Page Scan Type",			105 },
 	{ "Read AFH Channel Assessment Mode",		106 },
@@ -421,6 +434,7 @@ static hci_map commands_map[] = {
 	{ "Reserved",					109 },
 	{ "Reserved",					110 },
 	{ "Reserved",					111 },
+
 	{ "Reserved",					112 },
 	{ "Reserved",					113 },
 	{ "Reserved",					114 },
@@ -429,6 +443,7 @@ static hci_map commands_map[] = {
 	{ "Read Local Supported Features",		117 },
 	{ "Read Local Extended Features",		118 },
 	{ "Read Buffer Size",				119 },
+
 	{ "Read Country Code",				120 },
 	{ "Read BD ADDR",				121 },
 	{ "Read Failed Contact Counter",		122 },
@@ -437,6 +452,7 @@ static hci_map commands_map[] = {
 	{ "Read RSSI",					125 },
 	{ "Read AFH Channel Map",			126 },
 	{ "Read BD Clock",				127 },
+
 	{ "Read Loopback Mode",				128 },
 	{ "Write Loopback Mode",			129 },
 	{ "Enable Device Under Test Mode",		130 },
@@ -445,11 +461,43 @@ static hci_map commands_map[] = {
 	{ "Reject Synchronous Connection",		133 },
 	{ "Reserved",					134 },
 	{ "Reserved",					135 },
+
 	{ "Read Extended Inquiry Response",		136 },
 	{ "Write Extended Inquiry Response",		137 },
-	{ "Unknown",					138 },
-	{ "Unknown",					139 },
-	{ "Sniff Subrate",				140 },
+	{ "Refresh Encryption Key",			138 },
+	{ "Reserved",					139 },
+	{ "Sniff Subrating",				140 },
+	{ "Read Simple Pairing Mode",			141 },
+	{ "Write Simple Pairing Mode",			142 },
+	{ "Read Local OOB Data",			143 },
+
+	{ "Read Inquiry Transmit Power Level",		144 },
+	{ "Write Inquiry Transmit Power Level",		145 },
+	{ "Read Default Erroneous Data Reporting",	146 },
+	{ "Write Default Erroneous Data Reporting",	147 },
+	{ "Reserved",					148 },
+	{ "Reserved",					149 },
+	{ "Reserved",					150 },
+	{ "IO Capability Request Reply",		151 },
+
+	{ "User Confirmation Request Reply",		152 },
+	{ "User Confirmation Request Negative Reply",	153 },
+	{ "User Passkey Request Reply",			154 },
+	{ "User Passkey Request Negative Reply",	155 },
+	{ "Remote OOB Data Request Reply",		156 },
+	{ "Write Simple Pairing Debug Mode",		157 },
+	{ "Enhanced Flush",				158 },
+	{ "Remote OOB Data Request Negative Reply",	159 },
+
+	{ "Reserved",					160 },
+	{ "Reserved",					161 },
+	{ "Send Keypress Notification",			162 },
+	{ "IO Capabilities Response Negative Reply",	163 },
+	{ "Reserved",					164 },
+	{ "Reserved",					165 },
+	{ "Reserved",					166 },
+	{ "Reserved",					167 },
+
 	{ NULL }
 };
 
@@ -467,7 +515,7 @@ char *hci_commandstostr(uint8_t *commands, char *pref, int width)
 	m = commands_map;
 
 	while (m->str) {
-		if (hci_test_bit(m->val, commands))
+		if (commands[m->val / 8] & (1 << (m->val % 8)))
 			size += strlen(m->str) + (pref ? strlen(pref) : 0) + 3;
 		m++;
 	}
@@ -486,7 +534,7 @@ char *hci_commandstostr(uint8_t *commands, char *pref, int width)
 	m = commands_map;
 
 	while (m->str) {
-		if (hci_test_bit(m->val, commands)) {
+		if (commands[m->val / 8] & (1 << (m->val % 8))) {
 			if (strlen(off) + strlen(m->str) > width - 3) {
 				ptr += sprintf(ptr, "\n%s", pref ? pref : "");
 				off = ptr;
@@ -505,6 +553,7 @@ static hci_map ver_map[] = {
 	{ "1.1",	0x01 },
 	{ "1.2",	0x02 },
 	{ "2.0",	0x03 },
+	{ "2.1",	0x04 },
 	{ NULL }
 };
 
@@ -597,16 +646,16 @@ static hci_map lmp_features_map[8][9] = {
 		{ "<extended inquiry>",	LMP_EXT_INQ	},	/* Bit 0 */
 		{ "<no. 49>",		0x02		},	/* Bit 1 */
 		{ "<no. 50>",		0x04		},	/* Bit 2 */
-		{ "<no. 51>",		0x08		},	/* Bit 3 */
-		{ "<no. 52>",		0x10		},	/* Bit 4 */
-		{ "<no. 53>",		0x20		},	/* Bit 5 */
-		{ "<no. 54>",		0x40		},	/* Bit 6 */
+		{ "<simple pairing>",	LMP_SIMPLE_PAIR	},	/* Bit 3 */
+		{ "<encapsulated PDU>",	LMP_ENCAPS_PDU	},	/* Bit 4 */
+		{ "<err. data report>",	LMP_ERR_DAT_REP	},	/* Bit 5 */
+		{ "<non-flush flag>",	LMP_NFLUSH_PKTS	},	/* Bit 6 */
 		{ "<no. 55>",		0x80		},	/* Bit 7 */
 		{ NULL }
 	},
 	{	/* Byte 7 */
 		{ "<LSTO>",		LMP_LSTO	},	/* Bit 1 */
-		{ "<no. 57>",		0x02		},	/* Bit 1 */
+		{ "<inquiry TX power>",	LMP_INQ_TX_PWR	},	/* Bit 1 */
 		{ "<no. 58>",		0x04		},	/* Bit 2 */
 		{ "<no. 59>",		0x08		},	/* Bit 3 */
 		{ "<no. 60>",		0x10		},	/* Bit 4 */
@@ -960,7 +1009,9 @@ int hci_send_req(int dd, struct hci_request *r, int to)
 	try = 10;
 	while (try--) {
 		evt_cmd_complete *cc;
-		evt_cmd_status   *cs;
+		evt_cmd_status *cs;
+		evt_remote_name_req_complete *rn;
+		remote_name_req_cp *cp;
 
 		if (to) {
 			struct pollfd p;
@@ -1025,6 +1076,20 @@ int hci_send_req(int dd, struct hci_request *r, int to)
 			memcpy(r->rparam, ptr, r->rlen);
 			goto done;
 
+		case EVT_REMOTE_NAME_REQ_COMPLETE:
+			if (hdr->evt != r->event)
+				break;
+
+			rn = (void *) ptr;
+			cp = r->cparam;
+
+			if (bacmp(&rn->bdaddr, &cp->bdaddr))
+				continue;
+
+			r->rlen = MIN(len, r->rlen);
+			memcpy(r->rparam, ptr, r->rlen);
+			goto done;
+
 		default:
 			if (hdr->evt != r->event)
 				break;
@@ -2011,6 +2076,132 @@ int hci_write_ext_inquiry_response(int dd, uint8_t fec, uint8_t *data, int to)
 	return 0;
 }
 
+int hci_read_simple_pairing_mode(int dd, uint8_t *mode, int to)
+{
+	read_simple_pairing_mode_rp rp;
+	struct hci_request rq;
+
+	memset(&rq, 0, sizeof(rq));
+	rq.ogf    = OGF_HOST_CTL;
+	rq.ocf    = OCF_READ_SIMPLE_PAIRING_MODE;
+	rq.rparam = &rp;
+	rq.rlen   = READ_SIMPLE_PAIRING_MODE_RP_SIZE;
+
+	if (hci_send_req(dd, &rq, to) < 0)
+		return -1;
+
+	if (rp.status) {
+		errno = EIO;
+		return -1;
+	}
+
+	*mode = rp.mode;
+	return 0;
+}
+
+int hci_write_simple_pairing_mode(int dd, uint8_t mode, int to)
+{
+	write_simple_pairing_mode_cp cp;
+	write_simple_pairing_mode_rp rp;
+	struct hci_request rq;
+
+	memset(&cp, 0, sizeof(cp));
+	cp.mode = mode;
+
+	memset(&rq, 0, sizeof(rq));
+	rq.ogf    = OGF_HOST_CTL;
+	rq.ocf    = OCF_WRITE_SIMPLE_PAIRING_MODE;
+	rq.cparam = &cp;
+	rq.clen   = WRITE_SIMPLE_PAIRING_MODE_CP_SIZE;
+	rq.rparam = &rp;
+	rq.rlen   = WRITE_SIMPLE_PAIRING_MODE_RP_SIZE;
+
+	if (hci_send_req(dd, &rq, to) < 0)
+		return -1;
+
+	if (rp.status) {
+		errno = EIO;
+		return -1;
+	}
+
+	return 0;
+}
+
+int hci_read_local_oob_data(int dd, uint8_t *hash, uint8_t *randomizer, int to)
+{
+	read_local_oob_data_rp rp;
+	struct hci_request rq;
+
+	memset(&rq, 0, sizeof(rq));
+	rq.ogf    = OGF_HOST_CTL;
+	rq.ocf    = OCF_READ_LOCAL_OOB_DATA;
+	rq.rparam = &rp;
+	rq.rlen   = READ_LOCAL_OOB_DATA_RP_SIZE;
+
+	if (hci_send_req(dd, &rq, to) < 0)
+		return -1;
+
+	if (rp.status) {
+		errno = EIO;
+		return -1;
+	}
+
+	memcpy(hash, rp.hash, 16);
+	memcpy(randomizer, rp.randomizer, 16);
+	return 0;
+}
+
+int hci_read_inquiry_transmit_power_level(int dd, int8_t *level, int to)
+{
+	read_inquiry_transmit_power_level_rp rp;
+	struct hci_request rq;
+
+	memset(&rq, 0, sizeof(rq));
+	rq.ogf    = OGF_HOST_CTL;
+	rq.ocf    = OCF_READ_INQUIRY_TRANSMIT_POWER_LEVEL;
+	rq.rparam = &rp;
+	rq.rlen   = READ_INQUIRY_TRANSMIT_POWER_LEVEL_RP_SIZE;
+
+	if (hci_send_req(dd, &rq, to) < 0)
+		return -1;
+
+	if (rp.status) {
+		errno = EIO;
+		return -1;
+	}
+
+	*level = rp.level;
+	return 0;
+}
+
+int hci_write_inquiry_transmit_power_level(int dd, int8_t level, int to)
+{
+	write_inquiry_transmit_power_level_cp cp;
+	write_inquiry_transmit_power_level_rp rp;
+	struct hci_request rq;
+
+	memset(&cp, 0, sizeof(cp));
+	cp.level = level;
+
+	memset(&rq, 0, sizeof(rq));
+	rq.ogf    = OGF_HOST_CTL;
+	rq.ocf    = OCF_WRITE_INQUIRY_TRANSMIT_POWER_LEVEL;
+	rq.cparam = &cp;
+	rq.clen   = WRITE_INQUIRY_TRANSMIT_POWER_LEVEL_CP_SIZE;
+	rq.rparam = &rp;
+	rq.rlen   = WRITE_INQUIRY_TRANSMIT_POWER_LEVEL_RP_SIZE;
+
+	if (hci_send_req(dd, &rq, to) < 0)
+		return -1;
+
+	if (rp.status) {
+		errno = EIO;
+		return -1;
+	}
+
+	return 0;
+}
+
 int hci_read_transmit_power_level(int dd, uint16_t handle, uint8_t type, int8_t *level, int to)
 {
 	read_transmit_power_level_cp cp;
